'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var install = require('./install');
var semver = require('semver');
var io = require('inquirer');
var log = require('../../../lib/utils/log');
var Package = require('../../../lib/utils/package');

module.exports = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
    var isValid, pkg, reports, build, _ref2, result, _ref3, _result, deps, mioxCore, _ref4, _result2, nodeSass, _ref5, _result3, _ref6, _result4;

    return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
            switch (_context.prev = _context.next) {
                case 0:
                    isValid = true;

                    // package.json

                    if (Package.hasPackageJSON()) {
                        _context.next = 3;
                        break;
                    }

                    return _context.abrupt('return', log.error('当前目录不存在package.json文件'));

                case 3:
                    pkg = Package.read();
                    reports = [];

                    // build scripts

                    if (!(pkg.scripts && pkg.scripts.build)) {
                        _context.next = 24;
                        break;
                    }

                    build = pkg.scripts.build;

                    if (!/\bwebpack\b/.test(build)) {
                        _context.next = 20;
                        break;
                    }

                    if (!/ --progress/.test(build)) {
                        _context.next = 14;
                        break;
                    }

                    _context.next = 11;
                    return io.prompt([{
                        type: 'confirm',
                        name: 'result',
                        message: '开启`--progress`将导致无法通过在线编译，是否自动修复?',
                        default: true
                    }]);

                case 11:
                    _ref2 = _context.sent;
                    result = _ref2.result;

                    if (result) {
                        build = build.replace(/ --progress/, '');
                    } else {
                        isValid = false;
                        reports.push('不能开启`--progress`');
                    }

                case 14:
                    if (/ --hide-modules/.test(build)) {
                        _context.next = 20;
                        break;
                    }

                    _context.next = 17;
                    return io.prompt([{
                        type: 'confirm',
                        name: 'result',
                        message: '关闭`--hide-modules`将导致无法通过在线编译，是否自动修复?',
                        default: true
                    }]);

                case 17:
                    _ref3 = _context.sent;
                    _result = _ref3.result;

                    if (_result) {
                        build += ' --hide-modules';
                    } else {
                        isValid = false;
                        reports.push('不能关闭`--hide-modules`');
                    }

                case 20:
                    pkg.scripts.build = build;
                    Package.save(pkg);
                    _context.next = 26;
                    break;

                case 24:
                    isValid = false;
                    reports.push('不包含build脚本配置，无法在线编译');

                case 26:
                    if (!(pkg.dependencies || pkg.devDependencies || pkg.peerDependencies || pkg.optionalDependencies)) {
                        _context.next = 42;
                        break;
                    }

                    deps = Object.assign({}, pkg.dependencies, pkg.devDependencies, pkg.peerDependencies, pkg.optionalDependencies);
                    mioxCore = deps['miox-core'];

                    if (!(mioxCore && !semver.ltr('1.2.3', mioxCore))) {
                        _context.next = 35;
                        break;
                    }

                    _context.next = 32;
                    return io.prompt([{
                        type: 'confirm',
                        name: 'result',
                        message: '低版本的miox-core将导致无法通过在线编译，是否自动修复?',
                        default: true
                    }]);

                case 32:
                    _ref4 = _context.sent;
                    _result2 = _ref4.result;

                    if (_result2) {
                        install(['miox-core@1.2']);
                    } else {
                        isValid = false;
                        reports.push('miox-core的版本<=1.2.3');
                    }

                case 35:
                    nodeSass = deps['node-sass'];

                    if (!(nodeSass && !semver.ltr('4.5.2', nodeSass))) {
                        _context.next = 42;
                        break;
                    }

                    _context.next = 39;
                    return io.prompt([{
                        type: 'confirm',
                        name: 'result',
                        message: '低版本的node-sass将导致无法通过在线编译，是否自动修复?',
                        default: true
                    }]);

                case 39:
                    _ref5 = _context.sent;
                    _result3 = _ref5.result;

                    if (_result3) {
                        install(['node-sass@4.5']);
                    } else {
                        isValid = false;
                        reports.push('node-sass的版本<=4.5.2');
                    }

                case 42:
                    if (!(!Package.hasYarnLock() && !Package.hasNpmLock())) {
                        _context.next = 48;
                        break;
                    }

                    _context.next = 45;
                    return io.prompt([{
                        type: 'confirm',
                        name: 'result',
                        message: '当前项目下不存在锁文件，是否自动创建一个?',
                        default: true
                    }]);

                case 45:
                    _ref6 = _context.sent;
                    _result4 = _ref6.result;

                    if (_result4) {
                        install();
                    } else {
                        isValid = false;
                        reports.push('项目下不包含锁文件');
                    }

                case 48:

                    if (isValid) {
                        log.success('恭喜，检测通过，该分支可以发布');
                    } else {
                        log.error('该项目无法通过在线编译');
                        reports.forEach(function (log, i) {
                            console.log(`  ${i + 1}. ${log}`);
                        });
                    }

                case 49:
                case 'end':
                    return _context.stop();
            }
        }
    }, _callee, this);
}));