'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var path = require('path');
var encrypt = require('./encrypt');
var request = require('./request');
var configs = require('../configs');

/**
 * 实例一个Git用户
 * @param group: 'html5', 'web', 'titan' or a personal git account
 * @param token: git user's private token
 * @returns {Git}
 * @constructor
 */
var Git = function Git() {
    var group = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'html5';
    var token = arguments[1];

    if (!this instanceof Git) {
        return new Git(group, token);
    }

    if (configs.git.groups[group]) {
        this.isGroup = true;
        this.groupId = configs.git.groups[group].groupId;
    } else {
        this.isGroup = false;
        this.groupId = null;
    }

    this.group = group;
    this.prefix = configs.git.home;
    this.website = configs.git.web;

    Object.defineProperty(this, 'token', {
        get() {
            return {
                headers: {
                    'PRIVATE-TOKEN': token || encrypt.decode(configs.git.key)
                }
            };
        }
    });
};

Git.default = new Git();

Git.prototype.createProject = function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee(name, description, options) {
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        _context.next = 2;
                        return request.post(`${this.prefix}/projects`, Object.assign({
                            name: name,
                            description: description,
                            namespace_id: this.groupId,
                            visibility_level: 0,
                            approvals_before_merge: 1
                        }, options), this.token);

                    case 2:
                        return _context.abrupt('return', _context.sent);

                    case 3:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function (_x2, _x3, _x4) {
        return _ref.apply(this, arguments);
    };
}();

Git.prototype.selectProject = function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee2(name) {
        var project_name;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context2.next = 3;
                        return request.get(`${this.prefix}/projects/${project_name}`, this.token);

                    case 3:
                        return _context2.abrupt('return', _context2.sent);

                    case 4:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, this);
    }));

    return function (_x5) {
        return _ref2.apply(this, arguments);
    };
}();

Git.prototype.updateProject = function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee3(name, description, options) {
        var project_name;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context3.next = 3;
                        return request.put(`${this.prefix}/projects/${project_name}`, Object.assign({
                            name: name,
                            description: description
                        }, options), this.token);

                    case 3:
                        return _context3.abrupt('return', _context3.sent);

                    case 4:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this);
    }));

    return function (_x6, _x7, _x8) {
        return _ref3.apply(this, arguments);
    };
}();

Git.prototype.removeProject = function () {
    var _ref4 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee4(name) {
        var project_name;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
            while (1) {
                switch (_context4.prev = _context4.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context4.next = 3;
                        return request.delete(`${this.prefix}/projects/${project_name}`, this.token);

                    case 3:
                        return _context4.abrupt('return', _context4.sent);

                    case 4:
                    case 'end':
                        return _context4.stop();
                }
            }
        }, _callee4, this);
    }));

    return function (_x9) {
        return _ref4.apply(this, arguments);
    };
}();

Git.prototype.forkProject = function () {
    var _ref5 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee5(name) {
        var project_name;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
            while (1) {
                switch (_context5.prev = _context5.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context5.next = 3;
                        return request.post(`${this.prefix}/projects/${project_name}/fork`, { namespace: this.group }, this.token);

                    case 3:
                        return _context5.abrupt('return', _context5.sent);

                    case 4:
                    case 'end':
                        return _context5.stop();
                }
            }
        }, _callee5, this);
    }));

    return function (_x10) {
        return _ref5.apply(this, arguments);
    };
}();

Git.prototype.createBranch = function () {
    var _ref6 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee6(name) {
        var branch = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'dev';
        var ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'master';
        var project_name;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
            while (1) {
                switch (_context6.prev = _context6.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context6.next = 3;
                        return request.post(`${this.prefix}/projects/${project_name}/repository/branches`, { branch, ref }, this.token);

                    case 3:
                        return _context6.abrupt('return', _context6.sent);

                    case 4:
                    case 'end':
                        return _context6.stop();
                }
            }
        }, _callee6, this);
    }));

    return function (_x13) {
        return _ref6.apply(this, arguments);
    };
}();

Git.prototype.removeBranch = function () {
    var _ref7 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee7(name) {
        var branch = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'dev';
        var project_name;
        return _regenerator2.default.wrap(function _callee7$(_context7) {
            while (1) {
                switch (_context7.prev = _context7.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context7.next = 3;
                        return request.delete(`${this.prefix}/projects/${project_name}/repository/branches/${branch}`, this.token);

                    case 3:
                        return _context7.abrupt('return', _context7.sent);

                    case 4:
                    case 'end':
                        return _context7.stop();
                }
            }
        }, _callee7, this);
    }));

    return function (_x15) {
        return _ref7.apply(this, arguments);
    };
}();

Git.prototype.protectBranch = function () {
    var _ref8 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee8(name, branch) {
        var project_name;
        return _regenerator2.default.wrap(function _callee8$(_context8) {
            while (1) {
                switch (_context8.prev = _context8.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context8.next = 3;
                        return request.put(`${this.prefix}/projects/${project_name}/repository/branches/${branch}/protect`, null, this.token);

                    case 3:
                        return _context8.abrupt('return', _context8.sent);

                    case 4:
                    case 'end':
                        return _context8.stop();
                }
            }
        }, _callee8, this);
    }));

    return function (_x16, _x17) {
        return _ref8.apply(this, arguments);
    };
}();

Git.prototype.unprotectBranch = function () {
    var _ref9 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee9(name, branch) {
        var project_name;
        return _regenerator2.default.wrap(function _callee9$(_context9) {
            while (1) {
                switch (_context9.prev = _context9.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context9.next = 3;
                        return request.put(`${this.prefix}/projects/${project_name}/repository/branches/${branch}/unprotect`, null, this.token);

                    case 3:
                        return _context9.abrupt('return', _context9.sent);

                    case 4:
                    case 'end':
                        return _context9.stop();
                }
            }
        }, _callee9, this);
    }));

    return function (_x18, _x19) {
        return _ref9.apply(this, arguments);
    };
}();

Git.prototype.listBranches = function () {
    var _ref10 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee10(name) {
        var project_name;
        return _regenerator2.default.wrap(function _callee10$(_context10) {
            while (1) {
                switch (_context10.prev = _context10.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context10.next = 3;
                        return request.get(`${this.prefix}/projects/${project_name}/repository/branches`, this.token);

                    case 3:
                        return _context10.abrupt('return', _context10.sent);

                    case 4:
                    case 'end':
                        return _context10.stop();
                }
            }
        }, _callee10, this);
    }));

    return function (_x20) {
        return _ref10.apply(this, arguments);
    };
}();

Git.prototype.appendProjectUser = function () {
    var _ref11 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee11(name, uid, access_level) {
        var project_name;
        return _regenerator2.default.wrap(function _callee11$(_context11) {
            while (1) {
                switch (_context11.prev = _context11.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context11.next = 3;
                        return request.post(`${this.prefix}/projects/${project_name}/members`, { user_id: uid, access_level: access_level }, this.token);

                    case 3:
                        return _context11.abrupt('return', _context11.sent);

                    case 4:
                    case 'end':
                        return _context11.stop();
                }
            }
        }, _callee11, this);
    }));

    return function (_x21, _x22, _x23) {
        return _ref11.apply(this, arguments);
    };
}();

Git.prototype.searchProjectUser = function () {
    var _ref12 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee12(name, uid) {
        var project_name;
        return _regenerator2.default.wrap(function _callee12$(_context12) {
            while (1) {
                switch (_context12.prev = _context12.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context12.next = 3;
                        return request.get(`${this.prefix}/projects/${project_name}/members/${uid}`, this.token);

                    case 3:
                        return _context12.abrupt('return', _context12.sent);

                    case 4:
                    case 'end':
                        return _context12.stop();
                }
            }
        }, _callee12, this);
    }));

    return function (_x24, _x25) {
        return _ref12.apply(this, arguments);
    };
}();

Git.prototype.updateProjectUser = function () {
    var _ref13 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee13(name, uid, access_level) {
        var project_name;
        return _regenerator2.default.wrap(function _callee13$(_context13) {
            while (1) {
                switch (_context13.prev = _context13.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context13.next = 3;
                        return request.put(`${this.prefix}/projects/${project_name}/members`, { user_id: uid, access_level: access_level }, this.token);

                    case 3:
                        return _context13.abrupt('return', _context13.sent);

                    case 4:
                    case 'end':
                        return _context13.stop();
                }
            }
        }, _callee13, this);
    }));

    return function (_x26, _x27, _x28) {
        return _ref13.apply(this, arguments);
    };
}();

Git.prototype.removeProjectUser = function () {
    var _ref14 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee14(name, uid) {
        var project_name;
        return _regenerator2.default.wrap(function _callee14$(_context14) {
            while (1) {
                switch (_context14.prev = _context14.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context14.next = 3;
                        return request.delete(`${this.prefix}/projects/${project_name}/members/${uid}`, this.token);

                    case 3:
                        return _context14.abrupt('return', _context14.sent);

                    case 4:
                    case 'end':
                        return _context14.stop();
                }
            }
        }, _callee14, this);
    }));

    return function (_x29, _x30) {
        return _ref14.apply(this, arguments);
    };
}();

Git.prototype.downloadArchive = function () {
    var _ref15 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee15(name, where, tag) {
        var project_name, filename, extension;
        return _regenerator2.default.wrap(function _callee15$(_context15) {
            while (1) {
                switch (_context15.prev = _context15.next) {
                    case 0:
                        project_name = this.resolveProject(name);

                        if (tag) {
                            filename = path.resolve(where, `${name}-${tag}.tar.gz`);
                            extension = `.tar.gz?ref=${tag}`;
                        } else {
                            filename = path.resolve(where, `${name}.tar.gz`);
                            extension = '';
                        }

                        _context15.next = 4;
                        return request.download(`${this.prefix}/projects/${project_name}/repository/archive${extension}`, filename, this.token);

                    case 4:
                    case 'end':
                        return _context15.stop();
                }
            }
        }, _callee15, this);
    }));

    return function (_x31, _x32, _x33) {
        return _ref15.apply(this, arguments);
    };
}();

/**
 * @typedef {Object} Project - Gitlab project information
 * @property {number} id
 * @property {string} http_url_to_repo
 * @property {string} web_url
 * @property {string} name
 * @property {string} name_with_namespace
 * @property {string} path
 * @property {string} path_with_namespace
 */

/**
 * 根据项目名称搜索项目，只返回完整匹配的项目
 * @param {string} name - 项目完整名称
 * @param {boolean=true} simple - 是否使用简单搜索结果
 * @returns {Promise.<Project>}
 */
Git.prototype.searchGroupProject = function () {
    var _ref16 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee16(name) {
        var simple = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
        var result, i;
        return _regenerator2.default.wrap(function _callee16$(_context16) {
            while (1) {
                switch (_context16.prev = _context16.next) {
                    case 0:
                        if (this.isGroup) {
                            _context16.next = 2;
                            break;
                        }

                        throw new Error('无法搜索指定用户的项目');

                    case 2:
                        _context16.next = 4;
                        return request(`${this.prefix}/groups/${this.groupId}/projects?simple=${simple}&search=${name}`, this.token);

                    case 4:
                        result = _context16.sent;

                        if (!(result && result.length)) {
                            _context16.next = 12;
                            break;
                        }

                        i = result.length;

                    case 7:
                        if (!i--) {
                            _context16.next = 12;
                            break;
                        }

                        if (!(result[i].name === name)) {
                            _context16.next = 10;
                            break;
                        }

                        return _context16.abrupt('return', result[i]);

                    case 10:
                        _context16.next = 7;
                        break;

                    case 12:
                    case 'end':
                        return _context16.stop();
                }
            }
        }, _callee16, this);
    }));

    return function (_x35) {
        return _ref16.apply(this, arguments);
    };
}();

/**
 * 根据项目名称搜索项目，返回一个列表
 * @param {string} name - 项目完整名称
 * @param {boolean=true} simple - 是否使用简单搜索结果
 * @returns {Promise.<Project>}
 */
Git.prototype.searchGroupProjectList = function () {
    var _ref17 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee17(name) {
        var simple = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
        return _regenerator2.default.wrap(function _callee17$(_context17) {
            while (1) {
                switch (_context17.prev = _context17.next) {
                    case 0:
                        if (this.isGroup) {
                            _context17.next = 2;
                            break;
                        }

                        throw new Error('无法搜索指定用户的项目');

                    case 2:
                        _context17.next = 4;
                        return request(`${this.prefix}/groups/${this.groupId}/projects?simple=${simple}&search=${name}`, this.token);

                    case 4:
                        return _context17.abrupt('return', _context17.sent);

                    case 5:
                    case 'end':
                        return _context17.stop();
                }
            }
        }, _callee17, this);
    }));

    return function (_x37) {
        return _ref17.apply(this, arguments);
    };
}();

Git.prototype.searchBranch = function () {
    var _ref18 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee18(name, branch) {
        var project_name, result, i;
        return _regenerator2.default.wrap(function _callee18$(_context18) {
            while (1) {
                switch (_context18.prev = _context18.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context18.next = 3;
                        return request(`${this.prefix}/projects/${project_name}/repository/branches`, this.token);

                    case 3:
                        result = _context18.sent;

                        if (!(result && result.length)) {
                            _context18.next = 11;
                            break;
                        }

                        i = result.length;

                    case 6:
                        if (!i--) {
                            _context18.next = 11;
                            break;
                        }

                        if (!(result[i].name === branch)) {
                            _context18.next = 9;
                            break;
                        }

                        return _context18.abrupt('return', result[i]);

                    case 9:
                        _context18.next = 6;
                        break;

                    case 11:
                    case 'end':
                        return _context18.stop();
                }
            }
        }, _callee18, this);
    }));

    return function (_x38, _x39) {
        return _ref18.apply(this, arguments);
    };
}();

Git.prototype.searchTags = function () {
    var _ref19 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee19(name) {
        var project_name, result;
        return _regenerator2.default.wrap(function _callee19$(_context19) {
            while (1) {
                switch (_context19.prev = _context19.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context19.next = 3;
                        return request(`${this.prefix}/projects/${project_name}/repository/tags`, this.token);

                    case 3:
                        result = _context19.sent;

                        if (!(result && result.length)) {
                            _context19.next = 6;
                            break;
                        }

                        return _context19.abrupt('return', result);

                    case 6:
                    case 'end':
                        return _context19.stop();
                }
            }
        }, _callee19, this);
    }));

    return function (_x40) {
        return _ref19.apply(this, arguments);
    };
}();

Git.prototype.createTag = function () {
    var _ref20 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee20(name, tag, message) {
        var ref = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'master';
        var project_name;
        return _regenerator2.default.wrap(function _callee20$(_context20) {
            while (1) {
                switch (_context20.prev = _context20.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context20.next = 3;
                        return request.post(`${this.prefix}/projects/${project_name}/repository/tags`, {
                            tag_name: tag,
                            ref: ref,
                            message: message
                        }, this.token);

                    case 3:
                        return _context20.abrupt('return', _context20.sent);

                    case 4:
                    case 'end':
                        return _context20.stop();
                }
            }
        }, _callee20, this);
    }));

    return function (_x42, _x43, _x44) {
        return _ref20.apply(this, arguments);
    };
}();

Git.prototype.createRelease = function () {
    var _ref21 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee21(name, tag, description) {
        var project_name;
        return _regenerator2.default.wrap(function _callee21$(_context21) {
            while (1) {
                switch (_context21.prev = _context21.next) {
                    case 0:
                        project_name = this.resolveProject(name);
                        _context21.next = 3;
                        return request.post(`${this.prefix}/projects/${project_name}/repository/tags/${tag}/release`, {
                            tag_name: tag,
                            description: description // 支持markdown
                        }, this.token);

                    case 3:
                        return _context21.abrupt('return', _context21.sent);

                    case 4:
                    case 'end':
                        return _context21.stop();
                }
            }
        }, _callee21, this);
    }));

    return function (_x45, _x46, _x47) {
        return _ref21.apply(this, arguments);
    };
}();

/**
 * @typedef {Object} Member - Gitlab member information
 * @property {number} id
 * @property {string} username
 * @property {string} name
 * @property {string} state
 * @property {string} avatar_url
 * @property {string} web_url
 * @property {string} private_token
 * @property {boolean} can_create_group
 * @property {boolean} can_create_project
 */

/**
 * 根据用户名搜索用户，只返回完整匹配的用户
 * @param {string} username - 完整的用户名
 * @returns {Promise.<Member>}
 */
Git.prototype.searchUser = function () {
    var _ref22 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee22(username) {
        var result, i;
        return _regenerator2.default.wrap(function _callee22$(_context22) {
            while (1) {
                switch (_context22.prev = _context22.next) {
                    case 0:
                        _context22.next = 2;
                        return request(`${this.prefix}/users?username=${username}`, this.token);

                    case 2:
                        result = _context22.sent;

                        if (!(result && result.length)) {
                            _context22.next = 10;
                            break;
                        }

                        i = result.length;

                    case 5:
                        if (!i--) {
                            _context22.next = 10;
                            break;
                        }

                        if (!(result[i].username === username)) {
                            _context22.next = 8;
                            break;
                        }

                        return _context22.abrupt('return', result[i]);

                    case 8:
                        _context22.next = 5;
                        break;

                    case 10:
                    case 'end':
                        return _context22.stop();
                }
            }
        }, _callee22, this);
    }));

    return function (_x48) {
        return _ref22.apply(this, arguments);
    };
}();

Git.prototype.readFile = function () {
    var _ref23 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee23(name, filename) {
        var branch = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'master';
        return _regenerator2.default.wrap(function _callee23$(_context23) {
            while (1) {
                switch (_context23.prev = _context23.next) {
                    case 0:
                        _context23.prev = 0;
                        _context23.next = 3;
                        return request(`${this.website}/${this.group}/${name}/raw/${branch}/${filename}`, this.token);

                    case 3:
                        return _context23.abrupt('return', _context23.sent);

                    case 6:
                        _context23.prev = 6;
                        _context23.t0 = _context23['catch'](0);
                        return _context23.abrupt('return', '');

                    case 9:
                    case 'end':
                        return _context23.stop();
                }
            }
        }, _callee23, this, [[0, 6]]);
    }));

    return function (_x50, _x51) {
        return _ref23.apply(this, arguments);
    };
}();

Git.prototype.login = function () {
    var _ref24 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee24(username, password) {
        return _regenerator2.default.wrap(function _callee24$(_context24) {
            while (1) {
                switch (_context24.prev = _context24.next) {
                    case 0:
                        _context24.next = 2;
                        return request.post(`${this.prefix}/session?login=${username}&password=${encodeURIComponent(password)}`);

                    case 2:
                        return _context24.abrupt('return', _context24.sent);

                    case 3:
                    case 'end':
                        return _context24.stop();
                }
            }
        }, _callee24, this);
    }));

    return function (_x52, _x53) {
        return _ref24.apply(this, arguments);
    };
}();

Git.prototype.getCurrentUser = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee25() {
    return _regenerator2.default.wrap(function _callee25$(_context25) {
        while (1) {
            switch (_context25.prev = _context25.next) {
                case 0:
                    _context25.next = 2;
                    return request(`${this.prefix}/user`, this.token);

                case 2:
                    return _context25.abrupt('return', _context25.sent);

                case 3:
                case 'end':
                    return _context25.stop();
            }
        }
    }, _callee25, this);
}));

Git.prototype.resolveProject = function (name) {
    if (/^\d+$/.test(name)) {
        return name;
    } else {
        return `${this.group}%2F${name}`;
    }
};

module.exports = Git;